/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    gtpasesRhoRac

Description
    Solving diffusion-reaction task for 5 equation phenomenological Rho-Rac model.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "ODE.H"
#include "ODESolver.H"
//#include "CVODESolver.H"
#include "phenomRhoRac5EqModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createChemistry.H"
#   include "createFields.H"
#   include "initContinuityErrs.H"
#   include "createChemistryScheme.H"
//#   include "readTimeControls.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nCalculating Rho-Rac model\n" << endl;

//#   include "CourantNo.H"
//#   include "setInitialDeltaT.H"

    while (runTime.loop())
    {
        Info << "Time = " << runTime.timeName() << nl << endl;

#	include "readPISOControls.H"
//#	include "readTimeControls.H"
//#	include "CourantNo.H"
//#	include "setDeltaT.H"

// Mass transport part

	Info << "Solving transport\n";

        for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
        {
            fvScalarMatrix RhoTEqn
            (
                fvm::ddt(RhoT)
              - fvm::laplacian(D_Rho, RhoT)
            );

            RhoTEqn.solve();

            fvScalarMatrix RhoDEqn
            (
                fvm::ddt(RhoD)
              - fvm::laplacian(D_Rho, RhoD)
            );

            RhoDEqn.solve();

            fvScalarMatrix RacTEqn
            (
                fvm::ddt(RacT)
              - fvm::laplacian(D_Rac, RacT)
            );

            RacTEqn.solve();

            fvScalarMatrix RacDEqn
            (
                fvm::ddt(RacD)
              - fvm::laplacian(D_Rac, RacD)
            );

            RacDEqn.solve();

        }

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;


// Chemical part

	Info << "Solving chemistry\n";
	dtEst = runTime.deltaT().value()/ode_n;	// initial step length for ODESolver
	Tstart = runTime.value()-runTime.deltaT().value();	// start time
	Tfinish = runTime.value();				// finish time

	// cycle over mesh cells to solve chemistry
 	forAll(RhoT, celli)
	{
	    phenomRhoRacSystem.coeffs()[0]=RhoD[celli];
	    phenomRhoRacSystem.coeffs()[1]=RhoT[celli];
	    phenomRhoRacSystem.coeffs()[2]=DIAI[celli];
	    phenomRhoRacSystem.coeffs()[3]=DIAA[celli];
	    phenomRhoRacSystem.coeffs()[4]=ROCKI[celli];
	    phenomRhoRacSystem.coeffs()[5]=ROCKA[celli];
	    phenomRhoRacSystem.coeffs()[6]=RacD[celli];
	    phenomRhoRacSystem.coeffs()[7]=RacT[celli];
	    phenomRhoRacSystem.coeffs()[8]=PAKI[celli];
	    phenomRhoRacSystem.coeffs()[9]=PAKA[celli];

	    /*switch (solverType)		// solving chemial step
	    {
		case 0:
		    solver->solve(Tstart,Tfinish,ode_eps,dtEst);
		    break;
		case 1:
		    solverCVODE.solve(Tstart,Tfinish);
		    break;
	    }*/
            solver->solve(Tstart,Tfinish,ode_eps,dtEst);

	    phenomRhoRacSystem.coeffs()[0]=max(0.0,phenomRhoRacSystem.coeffs()[0]);
	    phenomRhoRacSystem.coeffs()[1]=max(0.0,phenomRhoRacSystem.coeffs()[1]);
	    phenomRhoRacSystem.coeffs()[2]=max(0.0,phenomRhoRacSystem.coeffs()[2]);
	    phenomRhoRacSystem.coeffs()[3]=max(0.0,phenomRhoRacSystem.coeffs()[3]);
	    phenomRhoRacSystem.coeffs()[4]=max(0.0,phenomRhoRacSystem.coeffs()[4]);
	    phenomRhoRacSystem.coeffs()[5]=max(0.0,phenomRhoRacSystem.coeffs()[5]);
	    phenomRhoRacSystem.coeffs()[6]=max(0.0,phenomRhoRacSystem.coeffs()[6]);
	    phenomRhoRacSystem.coeffs()[7]=max(0.0,phenomRhoRacSystem.coeffs()[7]);
	    phenomRhoRacSystem.coeffs()[8]=max(0.0,phenomRhoRacSystem.coeffs()[8]);
	    phenomRhoRacSystem.coeffs()[9]=max(0.0,phenomRhoRacSystem.coeffs()[9]);

	    RhoD[celli]=phenomRhoRacSystem.coeffs()[0];
	    RhoT[celli]=phenomRhoRacSystem.coeffs()[1];
	    DIAI[celli]=phenomRhoRacSystem.coeffs()[2];
	    DIAA[celli]=phenomRhoRacSystem.coeffs()[3];
	    ROCKI[celli]=phenomRhoRacSystem.coeffs()[4];
	    ROCKA[celli]=phenomRhoRacSystem.coeffs()[5];
	    RacD[celli]=phenomRhoRacSystem.coeffs()[6];
	    RacT[celli]=phenomRhoRacSystem.coeffs()[7];
	    PAKI[celli]=phenomRhoRacSystem.coeffs()[8];
	    PAKA[celli]=phenomRhoRacSystem.coeffs()[9];

	}

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;

	runTime.write();
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
