/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    gtpasesRhoRac

Description
    Solving diffusion-reaction task for 5 equation phenomenological Rho-Rac model.

    !!! Add printing of last average values
    !!! new line symbols are not pasted

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "ODE.H"
#include "ODESolver.H"
//#include "CVODESolver.H"
#include "phenomRhoRac5EqModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createChemistry.H"
#   include "createFields.H"
#   include "createBD.H"
#   include "initContinuityErrs.H"
#   include "createChemistryScheme.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nCalculating bifurcation diagram for Rho-Rac model\n" << endl;
    int stepsCounter = 0;
    scalar maxSteps = nValues * 2 + 3;
    scalar deltaStepT = (runTime.endTime().value() - runTime.startTime().value()) / maxSteps;
    scalar nextStep = runTime.startTime().value() + deltaStepT;
    scalar varValue = minValue;
    scalar deltaValue = (maxValue - minValue)/(nValues + 1.0);
    scalar scale = 1;

    int timesCounter = 0;

    scalar totVol = 0;
    scalar intRhoT = 0;
    scalar intRhoD = 0;
    scalar intDIAA = 0;
    scalar intDIAI = 0;
    scalar intROCKA = 0;
    scalar intROCKI = 0;
    scalar intRacT = 0;
    scalar intRacD = 0;
    scalar intPAKA = 0;
    scalar intPAKI = 0;

    scalar avRhoT = 0;
    scalar avRhoD = 0;
    scalar avDIAA = 0;
    scalar avDIAI = 0;
    scalar avROCKA = 0;
    scalar avROCKI = 0;
    scalar avRacT = 0;
    scalar avRacD = 0;
    scalar avPAKA = 0;
    scalar avPAKI = 0;

    std::ostringstream writePath;
    writePath << runTime.path() << "/avValuesBD.txt";
    FILE* avValuesFile;
    avValuesFile=fopen(writePath.str().c_str(), "w");
    std::ostringstream writeHead;
    writeHead << "avRhoT" << "\t" << "avRhoD" << "\t" << "avDIAA" << "\t" << "avDIAI" << "\t" << "avROCKA" << "\t" << "avROCKI" << "\t" << "avRacT" <<  "\t" << "avRacD" <<  "\t" << "avPAKA"  <<  "\t" << "avPAKI" << endl;
    fputs(writeHead.str().c_str(), avValuesFile);


    while (runTime.loop())
    {
        Info << "Time = " << runTime.timeName() << nl << endl;
	if (runTime.timeOutputValue() >= nextStep)
	{
	    nextStep += deltaStepT;
	    stepsCounter++;
	    if (stepsCounter <= nValues+1)
	    {
	        scale = (varValue + deltaValue) / varValue;
		varValue += deltaValue;
	    } else {
	        scale = (varValue - deltaValue) / varValue;
		varValue += -deltaValue;
	    }

	    avRhoT = avRhoT / timesCounter;
	    avRhoD = avRhoD / timesCounter;
	    avDIAA = avDIAA / timesCounter;
	    avDIAI = avDIAI / timesCounter;
	    avROCKA = avROCKA / timesCounter;
	    avROCKI = avROCKI / timesCounter;
	    avRacT = avRacT / timesCounter;
	    avRacD = avRacD / timesCounter;
	    avPAKA = avPAKA / timesCounter;
	    avPAKI = avPAKI / timesCounter;

            std::ostringstream writeString;
            writeString << avRhoT << "\t" << avRhoD << "\t" << avDIAA << "\t" << avDIAI << "\t" << avROCKA << "\t" << avROCKI << "\t" << avRacT <<  "\t" << avRacD <<  "\t" << avPAKA  <<  "\t" << avPAKI << endl;
            Info << "Average values are: " << endl;
            Info << avRhoT << "\t" << avRhoD << "\t" << avDIAA << "\t" << avDIAI << "\t" << avROCKA << "\t" << avROCKI << "\t" << avRacT <<  "\t" << avRacD <<  "\t" << avPAKA  <<  "\t" << avPAKI << endl;
            Info << writeString;
            fputs(writeString.str().c_str(), avValuesFile);

	    avRhoT = 0;
	    avRhoD = 0;
	    avDIAA = 0;
	    avDIAI = 0;
	    avROCKA = 0;
	    avROCKI = 0;
	    avRacT = 0;
	    avRacD = 0;
	    avPAKA = 0;
	    avPAKI = 0;
	    timesCounter = 0;

	    Info << "Change value of reagent " << bifDiagReagent << " to " << varValue << endl;
	    switch (int(bifDiagReagent))
	    {
	        case 0:
		    forAll(RhoT, celli)
		    {
		        RhoT[celli] = scale*RhoT[celli];
		        RhoD[celli] = scale*RhoD[celli];
		    }
		    break;
	        case 1:
		    forAll(DIAA, celli)
		    {
		        DIAA[celli] = scale*DIAA[celli];
		        DIAI[celli] = scale*DIAI[celli];
		    }
		    break;
	        case 2:
		    forAll(ROCKA, celli)
		    {
		        ROCKA[celli] = scale*ROCKA[celli];
		        ROCKI[celli] = scale*ROCKI[celli];
		    }
		    break;
	        case 3:
		    forAll(RacT, celli)
		    {
		        RacT[celli] = scale*RacT[celli];
		        RacD[celli] = scale*RacD[celli];
		    }
		    break;
	        case 4:
		    forAll(PAKA, celli)
		    {
		        PAKA[celli] = scale*PAKA[celli];
		        PAKI[celli] = scale*PAKI[celli];
		    }
		    break;
	    }
	}

#	include "readPISOControls.H"

// Mass transport part

	Info << "Solving transport\n";

        for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
        {
            fvScalarMatrix RhoTEqn
            (
                fvm::ddt(RhoT)
              - fvm::laplacian(D_Rho, RhoT)
            );

            RhoTEqn.solve();

            fvScalarMatrix RhoDEqn
            (
                fvm::ddt(RhoD)
              - fvm::laplacian(D_Rho, RhoD)
            );

            RhoDEqn.solve();

            fvScalarMatrix RacTEqn
            (
                fvm::ddt(RacT)
              - fvm::laplacian(D_Rac, RacT)
            );

            RacTEqn.solve();

            fvScalarMatrix RacDEqn
            (
                fvm::ddt(RacD)
              - fvm::laplacian(D_Rac, RacD)
            );

            RacDEqn.solve();

        }

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;


// Chemical part

	Info << "Solving chemistry\n";
	dtEst = runTime.deltaT().value()/ode_n;	// initial step length for ODESolver
	Tstart = runTime.value()-runTime.deltaT().value();	// start time
	Tfinish = runTime.value();				// finish time

	// cycle over mesh cells to solve chemistry
 	forAll(RhoT, celli)
	{
	    phenomRhoRacSystem.coeffs()[0]=RhoD[celli];
	    phenomRhoRacSystem.coeffs()[1]=RhoT[celli];
	    phenomRhoRacSystem.coeffs()[2]=DIAI[celli];
	    phenomRhoRacSystem.coeffs()[3]=DIAA[celli];
	    phenomRhoRacSystem.coeffs()[4]=ROCKI[celli];
	    phenomRhoRacSystem.coeffs()[5]=ROCKA[celli];
	    phenomRhoRacSystem.coeffs()[6]=RacD[celli];
	    phenomRhoRacSystem.coeffs()[7]=RacT[celli];
	    phenomRhoRacSystem.coeffs()[8]=PAKI[celli];
	    phenomRhoRacSystem.coeffs()[9]=PAKA[celli];

	    /*switch (solverType)		// solving chemial step
	    {
		case 0:
		    solver->solve(Tstart,Tfinish,ode_eps,dtEst);
		    break;
		case 1:
		    solverCVODE.solve(Tstart,Tfinish);
		    break;
	    }*/
            solver->solve(Tstart,Tfinish,ode_eps,dtEst);

	    phenomRhoRacSystem.coeffs()[0]=max(0.0,phenomRhoRacSystem.coeffs()[0]);
	    phenomRhoRacSystem.coeffs()[1]=max(0.0,phenomRhoRacSystem.coeffs()[1]);
	    phenomRhoRacSystem.coeffs()[2]=max(0.0,phenomRhoRacSystem.coeffs()[2]);
	    phenomRhoRacSystem.coeffs()[3]=max(0.0,phenomRhoRacSystem.coeffs()[3]);
	    phenomRhoRacSystem.coeffs()[4]=max(0.0,phenomRhoRacSystem.coeffs()[4]);
	    phenomRhoRacSystem.coeffs()[5]=max(0.0,phenomRhoRacSystem.coeffs()[5]);
	    phenomRhoRacSystem.coeffs()[6]=max(0.0,phenomRhoRacSystem.coeffs()[6]);
	    phenomRhoRacSystem.coeffs()[7]=max(0.0,phenomRhoRacSystem.coeffs()[7]);
	    phenomRhoRacSystem.coeffs()[8]=max(0.0,phenomRhoRacSystem.coeffs()[8]);
	    phenomRhoRacSystem.coeffs()[9]=max(0.0,phenomRhoRacSystem.coeffs()[9]);

	    RhoD[celli]=phenomRhoRacSystem.coeffs()[0];
	    RhoT[celli]=phenomRhoRacSystem.coeffs()[1];
	    DIAI[celli]=phenomRhoRacSystem.coeffs()[2];
	    DIAA[celli]=phenomRhoRacSystem.coeffs()[3];
	    ROCKI[celli]=phenomRhoRacSystem.coeffs()[4];
	    ROCKA[celli]=phenomRhoRacSystem.coeffs()[5];
	    RacD[celli]=phenomRhoRacSystem.coeffs()[6];
	    RacT[celli]=phenomRhoRacSystem.coeffs()[7];
	    PAKI[celli]=phenomRhoRacSystem.coeffs()[8];
	    PAKA[celli]=phenomRhoRacSystem.coeffs()[9];

	}

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;


// Average values
	Info << "Calculating average values\n";

        totVol = 0;
        intRhoT = 0;
        intRhoD = 0;
        intDIAA = 0;
        intDIAI = 0;
        intROCKA = 0;
        intROCKI = 0;
        intRacT = 0;
        intRacD = 0;
        intPAKA = 0;
        intPAKI = 0;

	forAll(RhoT, celli)
	{
	    scalar meshV;
	    meshV = mesh.V()[celli];
	    totVol += meshV;
	    intRhoT += RhoT[celli]*meshV;
	    intRhoD += RhoD[celli]*meshV;
	    intDIAA += DIAA[celli]*meshV;
	    intDIAI += DIAI[celli]*meshV;
	    intROCKA += ROCKA[celli]*meshV;
	    intROCKI += ROCKI[celli]*meshV;
	    intRacT += RacT[celli]*meshV;
	    intRacD += RacD[celli]*meshV;
	    intPAKA += PAKA[celli]*meshV;
	    intPAKI += PAKI[celli]*meshV;
	}

	avRhoT += intRhoT / totVol;
	avRhoD += intRhoD / totVol;
	avDIAA += intDIAA / totVol;
	avDIAI += intDIAI / totVol;
	avROCKA += intROCKA / totVol;
	avROCKI += intROCKI / totVol;
	avRacT += intRacT / totVol;
	avRacD += intRacD / totVol;
	avPAKA += intPAKA / totVol;
	avPAKI += intPAKI / totVol;

	timesCounter++;

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;

	runTime.write();
    }

    fclose(avValuesFile);
    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
