/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Foam::phenomRhoRac5EqModel

Description
    Class for solving Rho-Rac phenomenological model.

\*---------------------------------------------------------------------------*/

#ifndef PHENOMRHORAC5EQMODEL_H
#define PHENOMRHORAC5EQMODEL_H

#include "ODE.H"


namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class phenomRhoRac5EqModel Declaration
\*---------------------------------------------------------------------------*/

class phenomRhoRac5EqModel
:
    public ODE
{
    // Private data

    // Constants of model

    scalar aDIARho_;
    scalar aROCKRho_;
    scalar vDIAa_;
    scalar vDIAi_;
    scalar kDIAi_;
    scalar vROCKa_;
    scalar vROCKi_;
    scalar kROCKi_;
    scalar vRhoGEF_;
    scalar kRhoGEF_;
    scalar vRhoGAP_;
    scalar kRhoGAP_;
    scalar vRacGEF_;
    scalar kRacGEF_;
    scalar vRacGAP_;
    scalar kRacGAP_;
    scalar kDIAa_;
    scalar kROCKa_;
    scalar vPAKa_;
    scalar kPAKa_;
    scalar vPAKi_;
    scalar kPAKi_;
    scalar aRhoDIA_;
    scalar kRhoDIA_;
    scalar aRacDIA_;
    scalar kRacDIA_;
    scalar aRacROCK_;
    scalar kRacROCK_;
    scalar kDIARho_;
    scalar kROCKRho_;
    scalar aRhoPAK_;
    scalar kRhoPAK_;
    scalar aPAKRac_;
    scalar kPAKRac_;
    scalar aRacPAK_;
    scalar kRacPAK_;

    //- ODE coefficients
    scalarField coeffs_;

    // Private Member Functions

        //- Disallow default bitwise copy construct
        phenomRhoRac5EqModel(const phenomRhoRac5EqModel&);

        //- Disallow default bitwise assignment
        void operator=(const phenomRhoRac5EqModel&);

public:

    // Constructors

        phenomRhoRac5EqModel(const dictionary& chemicalProperties);			// заполняем константы по словарю


    // Destructor
	~phenomRhoRac5EqModel();

    // Member Functions

        // ODE parameters

            //- Return number of equations
            virtual label nEqns() const
            {
                return 10;
            }

            //- Return reference to concentrations
            virtual scalarField& coeffs()
            {
                return coeffs_;
            }

            //- Return reference to concentrations
            virtual const scalarField& coeffs() const
            {
                return coeffs_;
            }

            //- Return derivatives
            virtual void derivatives
            (
                const scalar time,
                const scalarField& c,
                scalarField& dcdt
            ) const;

            //- Return Jacobian
            virtual void jacobian
            (
                const scalar time,
                const scalarField& c,
                scalarField& dfdt,
                scalarSquareMatrix& dfdc
            ) const;

            //- Update ODE after the solution, advancing by delta
            virtual void update(const scalar delta);

};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

#endif
